from logging import debug
from time import sleep

import zmq
from serial import EIGHTBITS, PARITY_NONE, STOPBITS_ONE

from pysomq._utility import bind_socket
from ._utility import listen_feedback_push


class DummySerial:
    _read_line = 'read'

    def __init__(self,
                 port=None,
                 baudrate=9600,
                 bytesize=EIGHTBITS,
                 parity=PARITY_NONE,
                 stopbits=STOPBITS_ONE,
                 timeout=None,
                 xonxoff=False,
                 rtscts=False,
                 write_timeout=None,
                 dsrdtr=False,
                 inter_byte_timeout=None,
                 exclusive=None,
                 **kwargs
                 ):
        self._timeout = 0
        self._written = None
        self._line_term = '\r\n'
        self._rate = 0.01
        debug(f'Setup DummySerial write pusher')
        self._mq_feedback = bind_socket(listen_feedback_push, zmq.PUSH)

    @property
    def timeout(self):
        return self._timeout

    @timeout.setter
    def timeout(self, value):
        self._timeout = value

    @property
    def termination(self):
        return self._line_term

    @termination.setter
    def termination(self, value):
        self._line_term = value

    @property
    def readline_msg(self):
        return self._read_line

    @property
    def write_msg(self):
        return self._written

    @property
    def rate(self):
        return self._rate

    @rate.setter
    def rate(self, value):
        self._rate = value

    def write(self, msg: bytes):
        self._mq_feedback.send(msg)

    def readline(self):
        debug(f'Sleep for {self._rate} seconds')
        sleep(self._rate)
        return f'{self.readline_msg}{self.termination}'.encode()



import sys
import os

from pysomq._utility import parse_pysomq_args
from pysomq.serial_server import SerialServer


def main(args=None):
    args = parse_pysomq_args(args)
    server = SerialServer(streaming_socket=args.streaming_socket, listening_socket=args.listening_socket,
                          port=args.serial_port,
                          baudrate=args.serial_baudrate)
    server.timeout = args.timeout
    server.name = 'SerialServer'
    server.start()
    print(f'Started SerialServer daemon with the following configuration:\n'
          f'Module process id: {os.getpid()}\n'
          f'Server process id: {server.pid}\n'
          f'Serial configuration: Port {args.serial_port}, Baudrate {args.serial_baudrate}, Timeout {args.timeout}\n'
          f'Streaming configuration: Socket "{args.streaming_socket}"\n'
          f'Listening configuration: Socket "{args.listening_socket}"')
    server.join()


if __name__ == '__main__':
    main(sys.argv[1:])



import unittest
from logging import debug
from time import sleep

import zmq

from pysomq import SerialServer
from pysomq._utility import subscribe_socket, connect_socket
from tests._utility import listen_socket, listen_feedback_pull, stream_socket, stream_subscribe


class TestSerialServer(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestSerialServer, self).__init__(*args, **kwargs)
        self._server = None

    def setUp(self) -> None:
        super(TestSerialServer, self).setUp()
        self._server = SerialServer(streaming_socket=stream_socket, listening_socket=listen_socket, timeout=0.001)
        self._server.start()
        while not self._server.is_alive():
            pass
        sleep(0.02)  # Wait until everything is initialized in run -> serial and threads
        debug(f'Server and subscriber setup ann running')

    def tearDown(self) -> None:
        super(TestSerialServer, self).tearDown()
        self._server.terminate()

    def test_streaming(self):
        to_test = 'read\r\n'
        mq_stream_receiver = subscribe_socket(stream_subscribe, topic=b'')
        debug(f'Trying to read from stream')
        received = mq_stream_receiver.recv()
        debug(f'Received from stream: {received}')
        self.assertEqual(to_test, received.decode())

    def test_listening(self):
        to_test = 'sent\r\n'
        mq_feedback = connect_socket(listen_feedback_pull, zmq.PULL)
        mq_config_sender = connect_socket(listen_socket, zmq.REQ)
        debug(f'Send {to_test} over configuration socket')
        mq_config_sender.send(to_test.encode())
        mq_config_sender.recv()
        debug(f'Trying to read from listening')
        received = mq_feedback.recv()
        debug(f'Received from stream: {received}')
        self.assertEqual(to_test, received.decode())

    def test_multi_listening(self):
        to_test1 = 'sent1\r\n'
        to_test2 = 'sent2\r\n'
        mq_feedback = connect_socket(listen_feedback_pull, zmq.PULL)
        mq_config_sender1 = connect_socket(listen_socket, zmq.REQ)
        mq_config_sender2 = connect_socket(listen_socket, zmq.REQ)
        debug(f'Send {to_test1} over configuration socket 1')
        mq_config_sender1.send(to_test1.encode())
        mq_config_sender1.recv()
        debug(f'Trying to read from listening')
        received1 = mq_feedback.recv()
        debug(f'Received from stream: {received1}')
        debug(f'Send {to_test2} over configuration socket 2')
        mq_config_sender2.send(to_test2.encode())
        mq_config_sender2.recv()
        debug(f'Trying to read from listening')
        received2 = mq_feedback.recv()
        debug(f'Received from stream: {received2}')
        self.assertEqual(to_test1, received1.decode())
        self.assertEqual(to_test2, received2.decode())


if __name__ == '__main__':
    unittest.main()



from logging import basicConfig, DEBUG
from sys import platform

basicConfig(level=DEBUG, format='%(asctime)s - %(levelname)s - %(message)s')

if platform == 'linux':
    stream_socket = 'ipc:///tmp/stream'
    listen_socket = 'ipc:///tmp/listen'
    stream_subscribe = stream_socket
    listen_feedback_push = 'ipc:///tmp/feedback'
    listen_feedback_pull = listen_feedback_push
else:
    stream_socket = 'tcp://*:5555'
    listen_socket = 'tcp://*:5556'
    stream_subscribe = 'tcp://localhost:5555'
    listen_feedback_push = 'tcp://*:5557'
    listen_feedback_pull = 'tcp://localhost:5557'



import sys
from logging import debug

import zmq

from pysomq._utility import subscribe_socket, connect_socket, parse_pysomq_args


class SerialClient(object):
    def __init__(self, streaming_socket: str = 'tcp://localhost:5555', listening_socket: str = 'tcp://localhost:5556',
                 timeout=1):
        self._mq_context: zmq.context = None
        self._mq_streaming: zmq.socket = None
        self._mq_config: zmq.socket = None
        self._timeout = 0
        self.timeout = timeout
        self._setup_zmq(listening_socket, streaming_socket)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def __del__(self):
        self._cleanup_zmq()

    def _setup_zmq(self, listening_socket, streaming_socket):
        debug('Setup zmq connections')
        self._mq_context = zmq.Context()
        self._mq_streaming = subscribe_socket(streaming_socket, topic=b'', context=self._mq_context)
        self._mq_streaming.setsockopt(zmq.CONFLATE, True)
        self._mq_streaming.setsockopt(zmq.IMMEDIATE, True)
        self._mq_streaming.sndhwm = 1
        self._mq_config = connect_socket(listening_socket, zmq.REQ, context=self._mq_context)
        self._mq_config.setsockopt(zmq.IMMEDIATE, True)
        self._mq_config.sndhwm = 1

    def _cleanup_zmq(self):
        debug('Cleanup zmq connections')
        self._mq_streaming.unsubscribe('')
        self._mq_streaming.close()
        self._mq_config.close()
        self._mq_context.term()

    @property
    def timeout(self):
        return self._timeout / 1000

    @timeout.setter
    def timeout(self, value):
        self._timeout = value * 1000

    def open(self):
        pass

    def close(self):
        pass

    def readline(self):
        events = self._mq_streaming.poll(timeout=self._timeout)
        if events == zmq.POLLIN:
            return self._mq_streaming.recv()
        else:
            raise TimeoutError

    def write(self, value: bytes):
        self._mq_config.send(value)
        self._mq_config.recv()


if __name__ == '__main__':
    args = parse_pysomq_args(sys.argv[1:])
    client = SerialClient(streaming_socket=args.streaming_socket, listening_socket=args.listening_socket,
                          timeout=args.timeout)
    while True:
        line = client.readline().decode()
        print(line)



from pysomq.serial_client import SerialClient
from pysomq.serial_server import SerialServer

__all__ = ['__version__', 'SerialServer', 'SerialClient']

try:
    from ._version import version as __version__
except ImportError:
    # broken installation, we don't even try
    # unknown only works because I do poor mans version compare
    __version__ = "unknown"



import argparse
import sys
from logging import debug

import zmq

_connect_list = [zmq.REQ, zmq.SUB, zmq.PULL]


def bind_socket(socket_connection: str, socket_type, context=None):
    if socket_type in _connect_list:
        raise ValueError(f'Cannot bind to socket type {socket_type}')
    context = zmq.Context() if context is None else context
    socket = context.socket(socket_type)
    socket.bind(socket_connection)
    debug(f'Bound socket type {socket_type} to {socket_connection}')
    return socket


def connect_socket(socket_connection: str, socket_type, context=None):
    if socket_type not in _connect_list:
        raise ValueError(f'Cannot connect to socket type {socket_type}')
    context = zmq.Context() if context is None else context
    socket = context.socket(socket_type)
    socket.connect(socket_connection)
    debug(f'Connected socket type {socket_type} to {socket_connection}')
    return socket


def subscribe_socket(socket_connection: str, topic=b'', context=None):
    socket = connect_socket(socket_connection, zmq.SUB, context)
    socket.setsockopt(zmq.LINGER, 0)
    socket.setsockopt(zmq.SUBSCRIBE, topic)
    return socket


def create_parser():
    parser = argparse.ArgumentParser(description='Bridge a serial port over zmq pub/sub')
    parser.add_argument('--serial-port', dest='serial_port', type=str,
                        default='', help='Set the serial port (default: /dev/ttyS0 for linux and COM1 for win32)')
    parser.add_argument('--serial-baudrate', dest='serial_baudrate', type=int, default=9600,
                        help='Set the serial baudrate (default: %(default)s)')
    parser.add_argument('--timeout', dest='timeout', type=int, default=1,
                        help='Set the timeout for serial port and zeromq polling (default: %(default)s)')
    parser.add_argument('--streaming-socket', dest='streaming_socket', type=str, default='tcp://*:5555',
                        help='Set the zeromq streaming socket (default: %(default)s)')
    parser.add_argument('--listening-socket', dest='listening_socket', type=str, default='tcp://*:5556',
                        help='Set the zeromq listening port (default: %(default)s)')
    return parser


def parse_pysomq_args(args=None):
    parser = create_parser()
    retrieved_args = parser.parse_args(args)
    if retrieved_args.serial_port == '':
        retrieved_args.serial_port = default_serial_port()
    return retrieved_args


def default_serial_port():
    port = None
    if sys.platform == 'linux' or sys.platform == 'unix':
        port = '/dev/ttyS0'
    elif sys.platform == 'win32':
        port = 'COM1'
    return port



import unittest
from logging import debug
from time import sleep

import zmq

from pysomq import SerialServer, SerialClient
from pysomq._utility import connect_socket
from tests._utility import listen_socket, listen_feedback_pull, stream_socket


class TestSerialClient(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestSerialClient, self).__init__(*args, **kwargs)
        self._server = None

    def setUp(self) -> None:
        super(TestSerialClient, self).setUp()
        self._server = SerialServer(stream_socket, listen_socket, timeout=0.001)
        self._server.start()
        while not self._server.is_alive():
            pass
        self._client = SerialClient(streaming_socket=stream_socket, listening_socket=listen_socket)
        sleep(0.02)  # Wait until everything is initialized in run -> serial and threads
        debug(f'Server and client setup ann running')

    def tearDown(self) -> None:
        super(TestSerialClient, self).tearDown()
        self._server.terminate()

    def test_streaming(self):
        to_test = 'read\r\n'
        debug(f'Trying to read from client')
        received = self._client.readline()
        debug(f'Received from client: {received}')
        self.assertEqual(to_test, received.decode())

    def test_listening(self):
        to_test = 'sent\r\n'
        mq_feedback = connect_socket(listen_feedback_pull, zmq.PULL)
        debug(f'Send {to_test} over client')
        self._client.write(to_test.encode())
        debug(f'Trying to read from feedback socket')
        received = mq_feedback.recv()
        debug(f'Received from feedback socket: {received}')
        self.assertEqual(to_test, received.decode())


if __name__ == '__main__':
    unittest.main()



import argparse
import sys
import unittest
from unittest import mock

from pysomq._utility import parse_pysomq_args

if sys.platform == 'linux':
    port = '/dev/ttyS0'
elif sys.platform == 'win32':
    port = 'COM1'
baudrate = 9600
timeout = 1
streaming_socket = 'tcp://*:5555'
listening_socket = 'tcp://*:5556'


class ParserTest(unittest.TestCase):

    @mock.patch('argparse.ArgumentParser.parse_args',
                return_value=argparse.Namespace(serial_port=port, serial_baudrate=baudrate, timeout=timeout,
                                                streaming_socket=streaming_socket, listening_socket=listening_socket))
    def test_defaults(self, mock_args):
        args = parse_pysomq_args()
        assert args.serial_port == port
        assert args.serial_baudrate == baudrate
        assert args.timeout == timeout
        assert args.streaming_socket == streaming_socket
        assert args.listening_socket == listening_socket
